package com.aselimov.NS.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.aselimov.NS.model.Notification;
import com.aselimov.NS.service.NotificationService;

@RestController
@RequestMapping("/api/notifications")
public class NotificationController {

	private final NotificationService notificationService;

	@Autowired
	public NotificationController(final NotificationService notificationService) {
		this.notificationService = notificationService;
	}

	@PostMapping
	public ResponseEntity<Notification> createNotification(@RequestBody final Notification notification) {
		final Notification savedNotification = notificationService.saveNotification(notification);
		return new ResponseEntity<Notification>(savedNotification, HttpStatus.CREATED);
	}

	@GetMapping("/health")
	public ResponseEntity<String> healthCheck() {
		return ResponseEntity.ok("Notification service running...");
	}

	@GetMapping
	public ResponseEntity<List<Notification>> getAllNotifications() {
		return ResponseEntity.ok(notificationService.getAllNotifications());
	}

	@GetMapping("/{id}")
	public ResponseEntity<Notification> getNotificationById(@PathVariable final String id) {
		return ResponseEntity.ok(notificationService.getNotificationByIdOrThrow(id));
	}

	@GetMapping("/status/{status}")
	public ResponseEntity<List<Notification>> getNotificationsByStatus(
			@PathVariable final Notification.NotificationStatus status) {
		return ResponseEntity.ok(notificationService.getNotificationsByStatus(status));
	}

	@PutMapping("/{id}/status")
	public ResponseEntity<Void> updateNotificationStatus(@PathVariable final String id,
			@RequestParam final Notification.NotificationStatus status) {
		notificationService.updateNotificationStatus(id, status);
		return ResponseEntity.ok().build();
	}

	@DeleteMapping("/{id}")
	public ResponseEntity<Void> deleteNotification(@PathVariable String id) {
		notificationService.deleteNotification(id);
		return ResponseEntity.noContent().build();
	}
}
