package com.aselimov.NS.service;

import java.time.LocalDateTime;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.aselimov.NS.exception.NotificationNotFoundException;
import com.aselimov.NS.model.Notification;
import com.aselimov.NS.repository.NotificationRepository;

@Service
public class NotificationService {
	private final NotificationRepository notificationRepository;

	@Autowired
	public NotificationService(NotificationRepository notificationRepository) {
		this.notificationRepository = notificationRepository;
	}

	public Notification saveNotification(Notification notification) {
		// Set timestamps if not present
		if (notification.getCreatedAt() == null) {
			notification.setCreatedAt(LocalDateTime.now());
		}
		notification.setUpdatedAt(LocalDateTime.now());

		// Ensure status is set
		if (notification.getStatus() == null) {
			notification.setStatus(Notification.NotificationStatus.PENDING);
		}

		return notificationRepository.save(notification);
	}

	public List<Notification> getAllNotifications() {
		return notificationRepository.findAll();
	}

	public Notification getNotificationByIdOrThrow(String id) {
		return notificationRepository.findById(id).orElseThrow(() -> new NotificationNotFoundException(id));
	}

	public List<Notification> getNotificationsByStatus(Notification.NotificationStatus status) {
		return notificationRepository.findByStatus(status);
	}

	public List<Notification> getNotificationsByType(Notification.NotificationType type) {
		return notificationRepository.findByType(type);
	}

	public List<Notification> getNotificationsByRecipient(String recipient) {
		return notificationRepository.findByRecipient(recipient);
	}

	public void updateNotificationStatus(String id, Notification.NotificationStatus status) {
		notificationRepository.findById(id).ifPresent(notification -> {
			notification.setStatus(status);
			notification.setUpdatedAt(LocalDateTime.now());
			notificationRepository.save(notification);
		});
	}

	public void deleteNotification(String id) {
		notificationRepository.deleteById(id);
	}
}
