package com.aselimov.NS.repository;

import static org.junit.jupiter.api.Assertions.assertEquals;

import java.time.LocalDateTime;
import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.data.mongo.DataMongoTest;
import org.springframework.test.context.ActiveProfiles;

import com.aselimov.NS.model.Notification;
import com.aselimov.NS.model.Notification.NotificationType;

@DataMongoTest
@ActiveProfiles("test")
public class NotificationRepositoryTest {

	@BeforeEach
	public void setup() {
		notificationRepository.deleteAll();
	}

	public NotificationRepository getNotificationRepository() {
		return notificationRepository;
	}

	@Autowired
	private NotificationRepository notificationRepository;

	@Test
	public void shouldSaveNotification() {
		// Given
		final Notification notification = new Notification();
		notification.setType(Notification.NotificationType.EMAIL);
		notification.setRecipient("test@example.com");
		notification.setSubject("Test Subject");
		notification.setContent("Test Content");
		notification.setStatus(Notification.NotificationStatus.PENDING);
		notification.setCreatedAt(LocalDateTime.now());
		notification.setUpdatedAt(LocalDateTime.now());

		// When
		Notification savedNotification = notificationRepository.save(notification);

		// Then
		assert (savedNotification.getId() != null) : "Notification shouldn't be null";
		assertEquals(Notification.NotificationType.EMAIL, savedNotification.getType());
		assertEquals("test@example.com", savedNotification.getRecipient());
	}

	@Test
	public void shouldFindByStatus() {
		// Given
		Notification notification1 = new Notification();
		notification1.setType(Notification.NotificationType.EMAIL);
		notification1.setRecipient("test1@example.com");
		notification1.setStatus(Notification.NotificationStatus.PENDING);
		notification1.setCreatedAt(LocalDateTime.now());
		notification1.setUpdatedAt(LocalDateTime.now());

		Notification notification2 = new Notification();
		notification2.setType(Notification.NotificationType.SMS);
		notification2.setRecipient("test2@example.com");
		notification2.setStatus(Notification.NotificationStatus.SENT);
		notification2.setCreatedAt(LocalDateTime.now());
		notification2.setUpdatedAt(LocalDateTime.now());

		notificationRepository.saveAll(List.of(notification1, notification2));

		// When
		List<Notification> pendingNotifications = notificationRepository
				.findByStatus(Notification.NotificationStatus.PENDING);

		// Then
		assertEquals(1, pendingNotifications.size());
		assertEquals("test1@example.com", pendingNotifications.get(0).getRecipient());
	}

	@Test
	public void shouldFindByType() {
		// Given
		Notification notification = new Notification();
		notification.setType(Notification.NotificationType.PUSH);
		notification.setRecipient("test@example.com");
		notification.setStatus(Notification.NotificationStatus.PENDING);
		notification.setCreatedAt(LocalDateTime.now());
		notification.setUpdatedAt(LocalDateTime.now());

		notificationRepository.save(notification);

		// When
		List<Notification> pushNotifications = notificationRepository.findByType(NotificationType.PUSH);

		// Then
		assertEquals(1, pushNotifications.size());
		assertEquals(NotificationType.PUSH, pushNotifications.get(0).getType());
	}
}
